import { EventEmitter, Component, ElementRef, ChangeDetectorRef, Input, Output, Directive, ComponentFactoryResolver, ViewContainerRef, HostListener, NgModule } from '@angular/core';
import { NG_VALUE_ACCESSOR, NgModel, FormsModule } from '@angular/forms';
import { getYear, format, isAfter, isBefore, isSameDay, addMonths, subMonths, setHours, startOfMonth, endOfMonth, setSeconds, setMinutes, eachDayOfInterval, getDate, getMonth, isSameMonth, isToday, getDay, subDays, setDay } from 'date-fns';
import { enUS } from 'date-fns/locale';
import { CommonModule } from '@angular/common';
import { Subscription } from 'rxjs';

function mergeCalendarOptions(opts) {
    return Object.assign(Object.assign({}, defaultOptions), opts);
}
const defaultOptions = {
    minYear: getYear(new Date()) - 30,
    maxYear: getYear(new Date()) + 30,
    format: 'yyyy/LL/dd',
    formatTitle: 'MMMM uuuu',
    formatDays: 'eeeeee',
    firstCalendarDay: 0,
    locale: enUS,
    closeOnSelected: false
};

class AirbnbCalendarComponent {
    constructor(elementRef, cd) {
        this.elementRef = elementRef;
        this.cd = cd;
        this.modelValue = new EventEmitter();
        this.fromValue = new EventEmitter();
        this.toValue = new EventEmitter();
        this.date = new Date();
        this.innerValue = null;
        this.isOpened = false;
        this.fromToDate = { from: null, to: null };
        this.onTouchedCallback = () => { };
        this.onChangeCallback = () => { };
    }
    get value() {
        return this.innerValue;
    }
    set value(val) {
        this.innerValue = val;
    }
    get controlsStatus() {
        return {
            from: Boolean(this.options.minYear && this.options.minYear < this.calendar.year && this.calendar.month !== 0),
            to: Boolean(this.options.maxYear && this.options.maxYear > this.calendarNext.year && this.calendarNext.month !== 11)
        };
    }
    writeValue(val) {
        this.innerValue = val;
    }
    registerOnChange(fn) {
        this.onChangeCallback = fn;
    }
    registerOnTouched(fn) {
        this.onTouchedCallback = fn;
    }
    ngOnInit() {
        this.options = mergeCalendarOptions(this.options);
        this.initCalendar();
    }
    ngOnChanges() {
        this.options = mergeCalendarOptions(this.options);
        this.initCalendar();
    }
    selectDay(index, calendar) {
        if (index) {
            const cal = calendar === 'primary' ? this.calendar : this.calendarNext;
            if (!this.fromToDate.from) {
                this.fromToDate.from = cal.days[index].date;
                const from = format(this.fromToDate.from, this.options.format);
                this.value = from;
                this.modelValue.next(from);
                this.fromValue.next(from);
            }
            else if (this.fromToDate.from && !this.fromToDate.to) {
                this.fromToDate.to = cal.days[index].date;
                const from = format(this.fromToDate.from, this.options.format);
                const to = format(this.fromToDate.to, this.options.format);
                this.value = `${from}-${to}`;
                this.modelValue.next(this.value);
                this.toValue.next(this.value);
                if (this.options.closeOnSelected) {
                    this.isOpened = false;
                }
            }
            else if (this.fromToDate.to) {
                this.fromToDate = { from: cal.days[index].date, to: null };
                const from = format(this.fromToDate.from, this.options.format);
                this.value = from;
                this.modelValue.next(from);
                this.fromValue.next(from);
            }
        }
        this.calendar.days = this.calendar.days.map((d) => {
            return Object.assign(Object.assign({}, d), {
                isIncluded: isAfter(d.date, this.fromToDate.from || new Date()) && isBefore(d.date, this.fromToDate.to || new Date()),
                isActive: isSameDay(this.fromToDate.from || new Date(), d.date) || isSameDay(this.fromToDate.to, d.date)
            });
        });
        this.calendarNext.days = this.calendarNext.days.map((d) => {
            return Object.assign(Object.assign({}, d), {
                isIncluded: isAfter(d.date, this.fromToDate.from || new Date()) && isBefore(d.date, this.fromToDate.to || new Date()),
                isActive: isSameDay(this.fromToDate.from || new Date(), d.date) || isSameDay(this.fromToDate.to, d.date)
            });
        });
    }
    nextMonth() {
        this.date = addMonths(this.date, 1);
        this.initCalendar();
        this.selectDay();
    }
    prevMonth() {
        this.date = subMonths(this.date, 1);
        this.initCalendar();
        this.selectDay();
    }
    initCalendar() {
        const date = new Date(this.date.getTime());
        this.calendar = this.generateCalendar(date);
        this.calendarNext = this.generateCalendar(addMonths(date, 1));
    }
    generateCalendar(date = new Date()) {
        const [start, end, now] = [
            setHours(startOfMonth(date), 0),
            setHours(endOfMonth(date), 0),
            setSeconds(setMinutes(setHours(new Date(), 0), 0), 0)
        ];
        const days = eachDayOfInterval({ start, end })
            .map(d => {
            d = setSeconds(setMinutes(setHours(d, 0), 0), 0);
            return {
                date: d,
                day: getDate(d),
                month: getMonth(d),
                year: getYear(d),
                isSameMonth: isSameMonth(d, start),
                isToday: isToday(d),
                isSelectable: isBefore(now, d) || isSameDay(now, d),
                isSelected: false,
                isVisible: true,
                isIncluded: isAfter(d, this.fromToDate.from || new Date()) && isBefore(d, this.fromToDate.to || new Date()),
                isActive: false
            };
        })
            .reduce((acc, curr, index, arr) => {
            const first = this.options.firstCalendarDay || 0;
            const tmp = getDay(start) - first;
            if (tmp > 0 && arr.length - 1 === index) {
                acc.unshift(...[...new Array(tmp)].map((_, i) => {
                    const curr = setSeconds(setMinutes(setHours(subDays(start, i + 1), 0), 0), 0);
                    return {
                        date: curr,
                        day: getDate(curr),
                        month: getMonth(curr),
                        year: getYear(curr),
                        isSameMonth: false,
                        isToday: false,
                        isSelectable: false,
                        isSelected: false,
                        isVisible: true,
                        isIncluded: isAfter(curr, this.fromToDate.from || new Date()) && isBefore(curr, this.fromToDate.to || new Date()),
                        isActive: false
                    };
                }));
            }
            return acc.concat(curr);
        }, [])
            .sort((a, b) => (a.date >= b.date ? 1 : -1));
        const dayNames = [];
        const dayStart = this.options.firstCalendarDay || 0;
        for (let i = dayStart; i <= 6 + dayStart; i++) {
            const date = setDay(new Date(), i);
            dayNames.push(format(date, this.options.formatDays || 'eeeeee', { locale: this.options.locale }));
        }
        return {
            month: getMonth(date),
            year: getYear(date),
            title: format(date, this.options.formatTitle || 'MMMM uuuu', { locale: this.options.locale }),
            days,
            dayNames
        };
    }
}
AirbnbCalendarComponent.decorators = [
    { type: Component, args: [{
                selector: 'airbnb-calendar',
                template: "<div class=\"airbnb-calendar-container\" *ngIf=\"isOpened\">\n  <div class=\"airbnb-calendar\">\n    <section class=\"calendar\">\n      <div class=\"controls\">\n        <svg\n          xmlns=\"http://www.w3.org/2000/svg\"\n          width=\"8\"\n          height=\"14\"\n          class=\"chevron-left\"\n          (click)=\"prevMonth()\"\n          [class.is-disabled]=\"!controlsStatus.from\"\n        >\n          <path d=\"M2.414 7l5.293 5.293a1 1 0 01-1.414 1.414l-6-6a1 1 0 010-1.414l6-6a1 1 0 011.414 1.414L2.414 7z\" />\n        </svg>\n        <span class=\"month-name\">{{ calendar.title }}</span>\n      </div>\n      <div class=\"calendar-day-names\">\n        <span *ngFor=\"let name of calendar.dayNames\" class=\"day\">{{ name }}</span>\n      </div>\n      <div class=\"calendar-days\">\n        <span\n          *ngFor=\"let day of calendar.days; let i = index\"\n          [ngClass]=\"{\n            day: true,\n            'is-prev-month': !day.isSameMonth,\n            'is-today': day.isToday,\n            'is-selected': day.isSelected,\n            'is-disabled':\n              !day.isSelectable ||\n              ({}.toString.call(this.options.maxDate) === '[object Date]' ? this.options.maxDate! < day.date : false) ||\n              ({}.toString.call(this.options.minDate) === '[object Date]' ? this.options.minDate! > day.date : false),\n            'is-included': day.isIncluded,\n            'is-invisible': !day.isVisible,\n            'is-active': day.isActive || fromToDate.from === day.date || fromToDate.to === day.date\n          }\"\n          (click)=\"selectDay(i, 'primary')\"\n        >\n          {{ day.day }}\n        </span>\n      </div>\n    </section>\n    <section class=\"calendar\">\n      <div class=\"controls\">\n        <span class=\"month-name\">{{ calendarNext.title }}</span>\n        <svg\n          xmlns=\"http://www.w3.org/2000/svg\"\n          width=\"8\"\n          height=\"14\"\n          class=\"chevron-right\"\n          (click)=\"nextMonth()\"\n          [class.is-disabled]=\"!controlsStatus.to\"\n        >\n          <path d=\"M5.586 7L.293 12.293a1 1 0 001.414 1.414l6-6a1 1 0 000-1.414l-6-6A1 1 0 00.293 1.707L5.586 7z\" />\n        </svg>\n      </div>\n      <div class=\"calendar-day-names\">\n        <span *ngFor=\"let name of calendarNext.dayNames\" class=\"day\">{{ name }}</span>\n      </div>\n      <div class=\"calendar-days\">\n        <span\n          *ngFor=\"let day of calendarNext.days; let i = index\"\n          class=\"day\"\n          [ngClass]=\"{\n            'is-disabled':\n              !day.isSelectable ||\n              ({}.toString.call(this.options.maxDate) === '[object Date]' ? this.options.maxDate! < day.date : false) ||\n              ({}.toString.call(this.options.minDate) === '[object Date]' ? this.options.minDate! > day.date : false),\n            'is-prev-month': !day.isSameMonth,\n            'is-today': day.isToday,\n            'is-selected': day.isSelected,\n            'is-included': day.isIncluded,\n            'is-invisible': !day.isVisible,\n            'is-active': day.isActive || fromToDate.from === day.date || fromToDate.to === day.date\n          }\"\n          (click)=\"selectDay(i, 'secondary')\"\n        >\n          {{ day.day }}\n        </span>\n      </div>\n    </section>\n  </div>\n</div>\n",
                providers: [{ provide: NG_VALUE_ACCESSOR, useExisting: AirbnbCalendarComponent, multi: true }],
                styles: [":host{box-sizing:border-box;display:block;position:relative;z-index:10}:host *,:host :after,:host :before{box-sizing:inherit}.airbnb-calendar-container{background:#fff;border-radius:4px;box-shadow:1px 2px 3px rgba(0,0,0,.05);display:block;height:380px;left:0;position:absolute;top:5px;width:850px}.airbnb-calendar-container .airbnb-calendar{border:1px solid #eaedf3;border-radius:4px;display:block;height:100%;width:100%}.airbnb-calendar-container .airbnb-calendar .calendar{float:left;height:100%;padding:40px 60px;width:50%}.airbnb-calendar-container .airbnb-calendar .calendar .controls{-moz-user-select:none;-ms-user-select:none;-webkit-user-select:none;align-items:center;display:flex;justify-content:center;margin-bottom:25px;position:relative;user-select:none;width:100%}.airbnb-calendar-container .airbnb-calendar .calendar .controls .month-name{font-size:15px;font-weight:600}.airbnb-calendar-container .airbnb-calendar .calendar .controls .chevron-left,.airbnb-calendar-container .airbnb-calendar .calendar .controls .chevron-right{cursor:pointer;height:14px;position:absolute;top:0;width:8px}.airbnb-calendar-container .airbnb-calendar .calendar .controls .chevron-left.is-disabled,.airbnb-calendar-container .airbnb-calendar .calendar .controls .chevron-right.is-disabled{cursor:not-allowed;pointer-events:none}.airbnb-calendar-container .airbnb-calendar .calendar .controls .chevron-left{left:0}.airbnb-calendar-container .airbnb-calendar .calendar .controls .chevron-right{right:0}.airbnb-calendar-container .airbnb-calendar .calendar .calendar-day-names{align-items:center;display:flex;margin-bottom:10px;width:100%}.airbnb-calendar-container .airbnb-calendar .calendar .calendar-day-names .day{-moz-user-select:none;-ms-user-select:none;-webkit-user-select:none;color:#717171;font-size:12px;font-weight:500;text-align:center;user-select:none;width:14.28571%}.airbnb-calendar-container .airbnb-calendar .calendar .calendar-days{display:block;width:100%}.airbnb-calendar-container .airbnb-calendar .calendar .calendar-days .day{-moz-user-select:none;-ms-user-select:none;-webkit-user-select:none;align-items:center;border:2px solid transparent;border-radius:50%;color:#222;display:flex;float:left;font-size:14px;font-weight:500;justify-content:center;margin:2px 0;position:relative;user-select:none;width:14.28571%}.airbnb-calendar-container .airbnb-calendar .calendar .calendar-days .day:before{content:\"\";display:block;padding-top:100%}.airbnb-calendar-container .airbnb-calendar .calendar .calendar-days .day:hover:not(.is-disabled){border-color:#222;border-radius:50%;cursor:pointer}.airbnb-calendar-container .airbnb-calendar .calendar .calendar-days .day.is-disabled{color:#d6d6d6;pointer-events:none}.airbnb-calendar-container .airbnb-calendar .calendar .calendar-days .day.is-prev-month{color:transparent}.airbnb-calendar-container .airbnb-calendar .calendar .calendar-days .day.is-active{background:#222;border-color:#222;color:#fff}.airbnb-calendar-container .airbnb-calendar .calendar .calendar-days .day.is-included{background:#f7f7f7;border-color:#f7f7f7;border-radius:0;color:#222}.airbnb-calendar-container .airbnb-calendar .calendar .calendar-days .day.is-invisible{visibility:hidden}"]
            },] }
];
AirbnbCalendarComponent.ctorParameters = () => [
    { type: ElementRef },
    { type: ChangeDetectorRef }
];
AirbnbCalendarComponent.propDecorators = {
    options: [{ type: Input }],
    modelValue: [{ type: Output }],
    fromValue: [{ type: Output }],
    toValue: [{ type: Output }]
};

class AirbnbCalendarDirective {
    constructor(cfr, vc, el, ngModel) {
        this.cfr = cfr;
        this.vc = vc;
        this.el = el;
        this.ngModel = ngModel;
        this.sub = new Subscription();
        this.componentFactory = this.cfr.resolveComponentFactory(AirbnbCalendarComponent);
        this.vc.clear();
        this.component = this.vc.createComponent(this.componentFactory);
        this.sub.add(this.component.instance.modelValue.subscribe((event) => {
            this.ngModel.control.patchValue(event);
        }));
        this.component.onDestroy(() => this.sub.unsubscribe());
    }
    ngOnChanges() {
        this.component.instance.options = mergeCalendarOptions(this.options);
        this.component.instance.cd.detectChanges();
    }
    onFocus() {
        this.component.instance.isOpened = true;
    }
    onBlurClick(e) {
        const container = this.el.nativeElement.parentElement.querySelector('.airbnb-calendar-container');
        const controls = this.el.nativeElement.parentElement.querySelectorAll('.controls');
        if (this.component.instance.fromToDate.from &&
            this.component.instance.fromToDate.to &&
            container &&
            e.target &&
            (container === e.target || container.contains(e.target)) &&
            [].findIndex.call(controls, (ctrl) => ctrl.contains(e.target)) === -1) {
            this.component.instance.fromToDate = { from: null, to: null };
            this.component.instance.value = null;
            this.component.instance.modelValue.next('');
            this.component.instance.fromValue.next('');
            this.component.instance.toValue.next('');
            this.component.instance.calendar.days = this.component.instance.calendar.days.map(d => (Object.assign(Object.assign({}, d), { isIncluded: false, isActive: false })));
            this.component.instance.calendarNext.days = this.component.instance.calendarNext.days.map(d => (Object.assign(Object.assign({}, d), { isIncluded: false, isActive: false })));
            this.component.instance.cd.detectChanges();
        }
        if (!this.component.instance.isOpened) {
            return;
        }
        const input = this.el.nativeElement;
        if (!input) {
            return;
        }
        if (e.target === input || input.contains(e.target)) {
            return;
        }
        if (container &&
            container !== e.target &&
            !e.target.classList.contains('day') &&
            !container.contains(e.target)) {
            this.component.instance.isOpened = false;
        }
    }
    onKeyup(ev) {
        if (!this.component.instance.isOpened) {
            return;
        }
        if (ev.keyCode === 27) {
            this.component.instance.isOpened = false;
            this.component.instance.cd.detectChanges();
            this.el.nativeElement.blur();
        }
    }
}
AirbnbCalendarDirective.decorators = [
    { type: Directive, args: [{
                selector: '[airbnb-calendar]'
            },] }
];
AirbnbCalendarDirective.ctorParameters = () => [
    { type: ComponentFactoryResolver },
    { type: ViewContainerRef },
    { type: ElementRef },
    { type: NgModel }
];
AirbnbCalendarDirective.propDecorators = {
    options: [{ type: Input }],
    onFocus: [{ type: HostListener, args: ['focus', ['$event.target'],] }],
    onBlurClick: [{ type: HostListener, args: ['document:click', ['$event'],] }],
    onKeyup: [{ type: HostListener, args: ['document:keyup', ['$event'],] }]
};

class AirbnbCalendarModule {
}
AirbnbCalendarModule.decorators = [
    { type: NgModule, args: [{
                declarations: [AirbnbCalendarComponent, AirbnbCalendarDirective],
                imports: [CommonModule, FormsModule],
                exports: [AirbnbCalendarComponent, AirbnbCalendarDirective],
                entryComponents: [AirbnbCalendarComponent]
            },] }
];

/**
 * Generated bundle index. Do not edit.
 */

export { AirbnbCalendarComponent, AirbnbCalendarModule, mergeCalendarOptions, AirbnbCalendarDirective as ɵa };
//# sourceMappingURL=ngx-airbnb-calendar.js.map
